// ============================================================================
// Copyright (c) 2022 by Terasic Technologies Inc.
// ============================================================================
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development 
//   Kits made by Terasic.  Other use of this code, including the selling 
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use 
//   or functionality of this code.
//
// ============================================================================
//           
//  Terasic Technologies Inc
//  9F., No.176, Sec.2, Gongdao 5th Rd, East Dist, Hsinchu City, 30070. Taiwan
//  
//  
//                     web: http://www.terasic.com/  
//                     email: support@terasic.com
//
// ============================================================================
//Date:  Mon Mar 21 14:16:53 2022
// ============================================================================


module MAX10_top(

      ///////// LED /////////
      output   [ 1: 0]   LED,

      ///////// iRX /////////
      input              iRX_MCLK,
      input              iRX_PCLK,
      input              iRX_SCLK,
      input              iRX_HS,
      input              iRX_VS,
      input    [15: 0]   iRX_RD,
      input    [15: 0]   iRX_GD,
      input    [15: 0]   iRX_BD,
      input              iRX_DE,
      input              iRX_INT,
      input              iRX_CS_N,
      input    [ 5: 0]   iRX_AP,
      input              iRX_RESET_N,

      ///////// MAX10 /////////
      input              MAX10_CLK1_50,

      ///////// iTX /////////
      input              iTX_MCLK,
      input              iTX_PCLK,
      input              iTX_SCK,
      input              iTX_HS,
      input              iTX_VS,
      input    [11: 0]   iTX_RD,
      input    [11: 0]   iTX_GD,
      input    [11: 0]   iTX_BD,
      input              iTX_DE,
      input              iTX_DSR3L,
      input              iTX_DSR3R,
      input    [ 3: 0]   iTX_I2S,
      input              iTX_INT,
      input              iTX_RST_N,
      input              iTX_SPDIF,
      input              iTX_WS,

      ///////// oRX /////////
      output             oRX_MCLK,
      output             oRX_PCLK,
      output             oRX_SCLK,
      output             oRX_HS,
      output             oRX_VS,
      output   [15: 0]   oRX_RD,
      output   [15: 0]   oRX_GD,
      output   [15: 0]   oRX_BD,
      output             oRX_DE,
      output             oRX_INT,
      output             oRX_CS_N,
      output   [ 5: 0]   oRX_AP,
      output             oRX_RESET_N,

      ///////// oTX /////////
      output             oTX_MCLK,
      output             oTX_PCLK,
      output             oTX_SCK,
      output             oTX_HS,
      output             oTX_VS,
      output   [11: 0]   oTX_RD,
      output   [11: 0]   oTX_GD,
      output   [11: 0]   oTX_BD,
      output             oTX_DE,
      output             oTX_DSR3L,
      output             oTX_DSR3R,
      output   [ 3: 0]   oTX_I2S,
      output             oTX_INT,
      output             oTX_RST_N,
      output             oTX_SPDIF,
      output             oTX_WS 

);


//=======================================================
//  REG/WIRE declarations
//=======================================================

//=======================================================
//  Structural coding
//=======================================================
//--1S generater 
wire  M50_1HZ ; 
wire  iTX_PCLK_1HZ,iRX_PCLK_1HZ ; 
CLOCKMEM  ck0( .RESET_n(1), .CLK(MAX10_CLK1_50) ,.CLK_FREQ ( 24_000_000  )  ,.CK_1HZ( M50_1HZ      ) ) ;
//CLOCKMEM  ck1( .RESET_n(1), .CLK(iTX_PCLK     ) ,.CLK_FREQ ( 297_000_000 )  ,.CK_1HZ( iTX_PCLK_1HZ ) ) ;
//CLOCKMEM  ck2( .RESET_n(1), .CLK(iRX_PCLK     ) ,.CLK_FREQ ( 148_500_000 )  ,.CK_1HZ( iRX_PCLK_1HZ ) ) ;

CLOCKMEM  ck2( .RESET_n(1), .CLK(oRX_PCLK     ) ,.CLK_FREQ ( 74_250_000 )  ,.CK_1HZ( iRX_PCLK_1HZ ) ) ;

//--by 1s counter 
FRE_MEASURE f_iTX_PCLK ( .RESET_n(1),.MES_CLK (oTX_PCLK)   , .HZ1(M50_1HZ) , .FREQ( ) ); 
FRE_MEASURE f_iRX_PCLK ( .RESET_n(1),.MES_CLK (iRX_PCLK)   , .HZ1(M50_1HZ) , .FREQ( ) ); 
FRE_MEASURE f_oRX_PCLK ( .RESET_n(1),.MES_CLK (oRX_PCLK)   , .HZ1(M50_1HZ) , .FREQ( ) ); 


///////////////////////////////////////
// HDMI-TX(include audio), SIL913
//wire locked ,iTX_PCLK_,iRX_PCLK_,iRX_PCLK__; 



// video:  FPGA --> TX 
assign oTX_PCLK   = iTX_PCLK; 
assign oTX_HS     = iTX_HS;   
assign oTX_VS     = iTX_VS;   
assign oTX_RD     = iTX_RD;   
assign oTX_GD     = iTX_GD;   
assign oTX_BD     = iTX_BD;   
assign oTX_DE     = iTX_DE;   


// audio: FPGA --> TX
assign oTX_I2S    = iTX_I2S;  
assign oTX_SPDIF  = iTX_SPDIF;
assign oTX_WS     = iTX_WS;   
assign oTX_SCK    = iTX_SCK;  
assign oTX_MCLK   = iTX_MCLK; 
assign oTX_DSR3L  = iTX_DSR3L;
assign oTX_DSR3R  = iTX_DSR3R; 


// control: FPGA --> TX
assign oTX_RST_N    = iTX_RST_N;
assign oTX_INT      = iTX_INT;

///////////////////////////////////////
// HDMI-RX, ADV7619
 
// video: RX --> FPGA
wire LOCK; 

wire CLOCK_74M25;
wire CLOCK_148M5;
wire CLOCK_132M ;
wire CLOCK_198M ;
wire CLOCK_99M ;

pll PL(
	.areset(0),
	.inclk0(MAX10_CLK1_50),
	.c0    (CLOCK_74M25),
	.c1    (CLOCK_148M5),
	.c2    (CLOCK_99M ),
	.c3    (CLOCK_132M ),
	.c4    (CLOCK_198M ),
	
	.locked(LOCK));

//assign  oRX_PCLK	= iRX_PCLK ;
	
//wire [2:0]ST ; 	
//SW SS(
//	.result(ST));
//
//assign oRX_PCLK	= (
//(ST==1)?CLOCK_74M25:(
//(ST==2)?CLOCK_148M5:(
//(ST==3)?CLOCK_297M :(//99
//(ST==4)?CLOCK_198M :(//132
//(ST==5)?CLOCK_99M  :(
//(ST==0)?iRX_PCLK   :0
//)))))) ;
//

wire oRX_PCLK_1 ,RESET_N_DY; 
wire [1:0]ST ; 
//assign ST={ iTX_WS ,iTX_SPDIF } ;
RESET_DELY de1( 
   .BUTTON_N (iRX_RESET_N & iTX_RST_N) , 
	.OSC_24    (iRX_PCLK ), 
   .RESET_N_DY(RESET_N_DY)  
) ; 
assign ST = (!RESET_N_DY )?1:3;

//assign oRX_PCLK_n	= ~oRX_PCLK;
assign oRX_PCLK= (

(ST==0)?CLOCK_74M25 :(
(ST==1)?CLOCK_148M5 :(
(ST==2)?CLOCK_99M   :(//99
(ST==3)?iRX_PCLK :iRX_PCLK
)))) ;

//TS T(
//.SW(ST)
//);

assign oRX_HS		= iRX_HS;
assign oRX_VS		= iRX_VS;
assign oRX_RD		= iRX_RD;
assign oRX_GD		= iRX_GD;
assign oRX_BD		= iRX_BD;
assign oRX_DE		= iRX_DE;

// audio: RX --> FPGA
assign oRX_SCLK	= iRX_SCLK;
assign oRX_MCLK	= iRX_MCLK;
assign oRX_AP		= iRX_AP;


// contorl: FPGA --> RX
assign oRX_CS_N		= iRX_CS_N;
assign oRX_INT			= iRX_INT;
assign oRX_RESET_N	= iRX_RESET_N;

assign LED = {iRX_PCLK_1HZ , iRX_PCLK_1HZ};//  ;//{1'b1,iRX_PCLK_1HZ } ; 
//


endmodule
